%define anolis_release 2

Name:          authselect
Version:       1.4.3
Release:       %{anolis_release}%{?dist}
Summary:       Configures authentication and identity sources from supported profiles
URL:           https://github.com/authselect/authselect

License:       GPLv3+
Source0:       %{url}/archive/refs/tags/%{version}.tar.gz
Source10:      authselect-lib-filelist


# Do not terminate build if language files are empty.
%define _empty_manifest_terminate_build 0

%bcond_without compat
%bcond_with user_nsswitch
%bcond_without enforce_authselect

%{?anolis:%global default_profile sssd}

BuildRequires: autoconf, automake, gcc, libtool
BuildRequires: m4, pkgconfig, pkgconfig(popt)
BuildRequires: gettext-devel, po4a, asciidoc, libcmocka-devel, libselinux-devel
%if %{with compat}
BuildRequires: python3-devel
%endif
Requires:      authselect-libs = %{version}-%{release}
Suggests:      sssd, samba-winbind, fprintd-pam, oddjob-mkhomedir

%description
Authselect is a tool to configure system identity and authentication sources and
providers by selecting a specific profile. Profile is a set of files that
describes how the resulting system configuration will look like. When a profile
is selected, authselect will create nsswitch.conf(5) and PAM(8) stack to use
identity and authentication sources defined by the profile.

%package libs
Summary:       Utility library used by the authselect tool
# Required by scriptlets
Requires:      sed, coreutils
Suggests:      systemd
%if %{with enforce_authselect}
# authselect now owns nsswitch.conf (glibc) and pam files
Conflicts:     pam < 1.5.2
Conflicts:     glibc < 2.34
# systemd, nss-mdns no longer contains nsswitch.conf scriptlets
Conflicts:     systemd < 250
Conflicts:     nss-mdns < 0.15.1
%endif

%description libs
Authselect is a tool to configure system identity and authentication sources and
providers by selecting a specific profile. Profile is a set of files that
describes how the resulting system configuration will look like. When a profile
is selected, authselect will create nsswitch.conf(5) and PAM(8) stack to use
identity and authentication sources defined by the profile.

This package provides common library files for authselect and used by the authselect
command line tool and any other potential front-ends.

%package devel
Summary:       Development libraries and headers for authselect
Requires:      authselect-libs = %{version}-%{release}

%description devel
Authselect is a tool to configure system identity and authentication sources and
providers by selecting a specific profile. Profile is a set of files that
describes how the resulting system configuration will look like. When a profile
is selected, authselect will create nsswitch.conf(5) and PAM(8) stack to use
identity and authentication sources defined by the profile.

This package provides system header files and development libraries for authselect.
Useful if you develop a front-end for the authselect library.

%if %{with compat}
%package compat
Summary:       Tool to provide minimum backwards compatibility with authconfig
Provides:      authconfig
Requires:      authselect = %{version}-%{release}
Recommends:    oddjob-mkhomedir
Suggests:      sssd, realmd, samba-winbind

%description compat
Authselect is a tool to configure system identity and authentication sources and
providers by selecting a specific profile. Profile is a set of files that
describes how the resulting system configuration will look like. When a profile
is selected, authselect will create nsswitch.conf(5) and PAM(8) stack to use
identity and authentication sources defined by the profile.

This package will replace %{_sbindir}/authconfig with a tool that will
translate some of the authconfig calls into authselect calls. It provides
only minimum backward compatibility and users are encouraged to migrate
to authselect completely.
%endif

%package doc
Summary:        Documentation files for %{name}
Requires:       %{name}-libs = %{version}-%{release}
BuildArch:      noarch

%description    doc
Authselect is a tool to configure system identity and authentication sources and
providers by selecting a specific profile. Profile is a set of files that
describes how the resulting system configuration will look like. When a profile
is selected, authselect will create nsswitch.conf(5) and PAM(8) stack to use
identity and authentication sources defined by the profile.

The %{name}-doc package contains documentation files for %{name}.

%package lang
Summary:        Language translations for authselect
Requires:       %{name}-libs = %{version}-%{release}
BuildArch:      noarch

%description lang
Authselect is a tool to configure system identity and authentication sources and
providers by selecting a specific profile. Profile is a set of files that
describes how the resulting system configuration will look like. When a profile
is selected, authselect will create nsswitch.conf(5) and PAM(8) stack to use
identity and authentication sources defined by the profile.

This package contains the language translation files for authselect

%prep
%autosetup -n %{name}-%{version}

%build
autoreconf -ifv
%configure \
%if %{with compat}
    --with-pythonbin="%{__python3}" \
    --with-compat \
%endif
%if %{with user_nsswitch}
    --with-user-nsswitch \
%endif
    %{nil}

%make_build

%check
%make_build check

%install
%make_install

# Find translations
%find_lang %{name}
%find_lang %{name} %{name}.8.lang --with-man
%find_lang %{name}-migration %{name}-migration.7.lang --with-man
%find_lang %{name}-profiles %{name}-profiles.5.lang --with-man

# We want this file to contain only manual page translations
%__sed -i '/LC_MESSAGES/d' %{name}.8.lang

# Remove .la and .a files created by libtool
find %{buildroot} \( -name "*.la" -o -name "*.a" \) -exec %__rm -vf {} \;

# Call ldconfig in %post/%postun
%ldconfig_scriptlets libs

# Generate compateibility abi/api files
%generate_compatibility_deps

%files libs -f %{SOURCE10}
%if %{with enforce_authselect}
%ghost %attr(0644,root,root) %{_sysconfdir}/nsswitch.conf
%ghost %attr(0644,root,root) %{_sysconfdir}/pam.d/fingerprint-auth
%ghost %attr(0644,root,root) %{_sysconfdir}/pam.d/password-auth
%ghost %attr(0644,root,root) %{_sysconfdir}/pam.d/postlogin
%ghost %attr(0644,root,root) %{_sysconfdir}/pam.d/smartcard-auth
%ghost %attr(0644,root,root) %{_sysconfdir}/pam.d/system-auth
%endif
%if %{with user_nsswitch}
%ghost %attr(0644,root,root) %{_sysconfdir}/authselect/user-nsswitch.conf
%ghost %attr(0644,root,root) %{_localstatedir}/lib/authselect/user-nsswitch-created
%endif

%files doc
%doc README.md

%files devel
%{_includedir}/authselect.h
%{_libdir}/libauthselect.so
%{_libdir}/pkgconfig/authselect.pc

%if %{with compat}
%files compat
%{_sbindir}/authconfig
%{python3_sitelib}/authselect/
%endif

%files lang -f %{name}.lang -f %{name}-profiles.5.lang -f %{name}.8.lang -f %{name}-migration.7.lang

%files
%{_bindir}/authselect
%{_mandir}/man8/authselect.8*
%{_mandir}/man7/authselect-migration.7*
%{_sysconfdir}/bash_completion.d/authselect-completion.sh

%global forcefile %{_sharedstatedir}/rpm-state/%{name}.force

%preun
if [ $1 == 0 ] ; then
    # Remove authselect symbolic links so all authselect files can be
    # deleted safely. If this fail, the uninstallation must fail to avoid
    # breaking the system by removing PAM files. However, the command can
    # only fail if it can not write to the file system.
    %{_bindir}/authselect opt-out
fi

%if %{with enforce_authselect}
%pre libs -p <lua>
force_file = rpm.expand("%{forcefile}")
authselect = rpm.expand("%{_bindir}/authselect")
os.remove(force_file)

-- Check if this is a new installation.
if tonumber(arg[2]) == 1
then
  f = io.open(force_file, "w")
  f:write("")
  f:close()
end

-- Check if we are upgrading from older version then authselect-1.3.0
-- The version command is not available on earlier versions
if tonumber(arg[2]) > 1
then
  comm = os.execute(authselect .. " check &> /dev/null")
  if comm ~= true
  then
    comm = os.execute(authselect .. " version &> /dev/null")
    if comm ~= true
    then
      f = io.open(force_file, "w")
      f:write("")
      f:close()
    end
  end
end

%else
%pre libs
exit 0
%endif

%posttrans libs
# Copy nsswitch.conf to user-nsswitch.conf if it was not yet created
%if %{with user_nsswitch}
if [ ! -f %{_sharedstatedir}/authselect/user-nsswitch-created ]; then
    %__cp -n %{_sysconfdir}/nsswitch.conf \
        %{_sysconfdir}/authselect/user-nsswitch.conf &> /dev/null
    touch %{_sharedstatedir}/authselect/user-nsswitch-created &> /dev/null
fi
%endif

# Keep nss-altfiles for all rpm-ostree based systems.
# See https://github.com/authselect/authselect/issues/48
if test -e /run/ostree-booted; then
    for PROFILE in `ls %{_datadir}/authselect/default`; do
        %{_bindir}/authselect create-profile $PROFILE --vendor --base-on $PROFILE \
            --symlink-pam --symlink-dconf --symlink=REQUIREMENTS --symlink=README &> /dev/null
%if %{with user_nsswitch}
        %__sed -ie "s/^\(passwd\|group\):\(.*\)systemd\(.*\)/\1:\2systemd altfiles\3/g" \
            %{_datadir}/authselect/vendor/$PROFILE/nsswitch.conf &> /dev/null
%else
        %__sed -ie 's/{if "with-altfiles":altfiles }/altfiles /g' \
            %{_datadir}/authselect/vendor/$PROFILE/nsswitch.conf &> /dev/null
%endif
    done
fi

%{_bindir}/authselect check &> /dev/null
if [ $? -eq 6 ]; then
    NOBACKUP="--nobackup"
fi

# If we are upgrading from pre authselect-1.3.0 or this is a new installation
# select the default configuration.
if [ -f %{forcefile} ]; then
    %{_bindir}/authselect select %{default_profile} --force $NOBACKUP &> /dev/null
    %__rm -f %{forcefile}
fi

# Apply any changes to profiles (validates configuration first internally)
%{_bindir}/authselect apply-changes &> /dev/null

exit 0

%changelog
* Fri May 31 2024 yanyihao <yan.yihao@zte.com.cn> - 1.4.3-2
- fix: su to a common user in chroot

* Tue Feb 27 2024 mgb01105731 <mgb01105731@alibaba-inc.com> - 1.4.3-1
- update to 1.4.3

* Fri Apr 14 2023 DengXiewei <dengxiewei@linux.alibaba.com> - 1.4.2-2
- optimise spec

* Mon Dec 26 2022 Kun(llfl) <llfl@linux.alibaba.com> - 1.4.2-1
- update to 1.4.2
- optimise spec

* Tue Sep 20 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 1.4.0-1
- update to 1.4.0

* Mon Mar 21 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 1.3.0-3
- optimise Conflicts version

* Wed Mar 16 2022 Mingyue Zhao <wb-zmy745940@alibaba-inc.com> - 1.3.0-2
- Set with_user_nsswitch=0 enforce_authselect=1

* Thu Mar 10 2022 Mingyue Zhao <wb-zmy745940@alibaba-inc.com> - 1.3.0-1
- Init for Anolis OS 23
